@@ -1,35 +1,36 @@
 use super::*;
 use crate::eval::EvalCtx;
+use either::Either;
 use std::borrow::Cow;
 
+use crate::utils::ValueExt;
 use serde_json::Value;
 
 fn flatten_recur<'a>(collect: &mut Vec<&'a Value>, a: &'a Value) {
     collect.push(a);
-    match a {
-        Value::Array(v) => v.iter().for_each(|a| flatten_recur(collect, a)),
-        Value::Object(m) => m.values().for_each(|a| flatten_recur(collect, a)),
-        _ => (),
-    }
+    a.iter().for_each(|a| flatten_recur(collect, a));
 }
 
 impl Path {
     pub(crate) fn has_parent(&self) -> bool {
         for op in &self.segments {
-            let result = matches!(
-                op,
+            let result = match op {
                 Segment::Dot(_, RawSelector::Parent(_))
                 | Segment::Recursive(_, Some(RawSelector::Parent(_)))
-                | Segment::Bracket(_, BracketSelector::Parent(_))
-            );
+                | Segment::Bracket(_, BracketSelector::Parent(_)) => true,
+                Segment::Bracket(_, BracketSelector::Path(p)) => p.has_parent(),
+                Segment::Bracket(_, BracketSelector::Filter(f)) => f.has_parent(),
+                _ => false,
+            };
+
             if result {
                 return true;
             }
         }
-        return false;
+        false
     }
 
-    pub(crate) fn eval(&self, ctx: &mut EvalCtx<'_>) {
+    pub(crate) fn eval(&self, ctx: &mut EvalCtx<'_, '_>) {
         for op in &self.segments {
             op.eval(ctx);
         }
@@ -43,7 +44,7 @@ impl Path {
 }
 
 impl Segment {
-    fn eval(&self, ctx: &mut EvalCtx<'_>) {
+    fn eval(&self, ctx: &mut EvalCtx<'_, '_>) {
         match self {
             Segment::Dot(_, op) => op.eval(ctx),
             Segment::Bracket(_, op) => op.eval(ctx),
@@ -62,13 +63,9 @@ impl Segment {
 }
 
 impl RawSelector {
-    fn eval(&self, ctx: &mut EvalCtx<'_>) {
+    fn eval(&self, ctx: &mut EvalCtx<'_, '_>) {
         match self {
-            RawSelector::Wildcard(_) => ctx.apply_matched(|_, a| match a {
-                Value::Array(v) => v.iter().collect(),
-                Value::Object(m) => m.values().collect(),
-                _ => vec![],
-            }),
+            RawSelector::Wildcard(_) => ctx.apply_matched(|_, a| a.iter()),
             RawSelector::Parent(_) => {
                 ctx.apply_matched(|ctx, a| ctx.parent_of(a));
             }
@@ -107,7 +104,7 @@ fn range(slice: &[Value], start: usize, end: usize) -> &[Value] {
 }
 
 impl StepRange {
-    fn eval(&self, ctx: &mut EvalCtx<'_>) {
+    fn eval(&self, ctx: &mut EvalCtx<'_, '_>) {
         let start = self.start.as_ref().map_or(0, |i| i.as_int());
         let end = self.end.as_ref().map_or(i64::MAX, |i| i.as_int());
         let step = self.step.as_ref().map_or(1, |i| i.as_int().get());
@@ -122,18 +119,18 @@ impl StepRange {
                 let iter = range(v, start, end).iter();
 
                 if rev {
-                    iter.rev().step_by(step).collect()
+                    Either::Left(iter.rev().step_by(step))
                 } else {
-                    iter.step_by(step).collect()
+                    Either::Right(iter.step_by(step))
                 }
             }
-            _ => vec![],
+            _ => Either::Right([].iter().step_by(1)),
         });
     }
 }
 
 impl Range {
-    fn eval(&self, ctx: &mut EvalCtx<'_>) {
+    fn eval(&self, ctx: &mut EvalCtx<'_, '_>) {
         let start = self.start.as_ref().map_or(0, |i| i.as_int());
         let end = self.end.as_ref().map_or(i64::MAX, |i| i.as_int());
 
@@ -142,15 +139,15 @@ impl Range {
                 let start = idx_handle(start, v).unwrap_or(0);
                 let end = idx_handle(end, v).unwrap_or(0);
 
-                range(v, start, end).iter().collect()
+                range(v, start, end)
             }
-            _ => vec![],
+            _ => &[],
         });
     }
 }
 
 impl UnionComponent {
-    fn eval(&self, ctx: &mut EvalCtx<'_>) {
+    fn eval(&self, ctx: &mut EvalCtx<'_, '_>) {
         match self {
             UnionComponent::StepRange(step_range) => step_range.eval(ctx),
             UnionComponent::Range(range) => range.eval(ctx),
@@ -171,24 +168,21 @@ impl UnionComponent {
 }
 
 impl BracketSelector {
-    fn eval(&self, ctx: &mut EvalCtx<'_>) {
+    fn eval(&self, ctx: &mut EvalCtx<'_, '_>) {
         match self {
             BracketSelector::Union(components) => {
                 let mut new_matched = Vec::new();
+                let old_matched = ctx.get_matched().to_owned();
                 for component in components {
-                    let mut new_ctx = ctx.child_ctx();
-                    component.eval(&mut new_ctx);
-                    new_matched.extend(new_ctx.into_matched());
+                    ctx.set_matched(old_matched.clone());
+                    component.eval(ctx);
+                    new_matched.extend(ctx.get_matched());
                 }
                 ctx.set_matched(new_matched);
             }
             BracketSelector::StepRange(step_range) => step_range.eval(ctx),
             BracketSelector::Range(range) => range.eval(ctx),
-            BracketSelector::Wildcard(_) => ctx.apply_matched(|_, a| match a {
-                Value::Array(v) => v.iter().collect(),
-                Value::Object(m) => m.values().collect(),
-                _ => vec![],
-            }),
+            BracketSelector::Wildcard(_) => ctx.apply_matched(|_, a| a.iter()),
             BracketSelector::Parent(_) => {
                 ctx.apply_matched(|ctx, a| ctx.parent_of(a));
             }
@@ -206,11 +200,10 @@ impl BracketSelector {
 }
 
 impl BracketLit {
-    fn eval(&self, ctx: &mut EvalCtx<'_>) {
+    fn eval(&self, ctx: &mut EvalCtx<'_, '_>) {
         match self {
             BracketLit::Int(i) => ctx.apply_matched(|_, a| match a {
-                Value::Array(v) => idx_handle(i.as_int(), v)
-                    .and_then(|idx| v.get(idx)),
+                Value::Array(v) => idx_handle(i.as_int(), v).and_then(|idx| v.get(idx)),
                 _ => None,
             }),
             BracketLit::String(s) => ctx.apply_matched(|_, a| match a {
@@ -222,7 +215,25 @@ impl BracketLit {
 }
 
 impl SubPath {
-    fn eval_expr<'a>(&self, ctx: &EvalCtx<'a>, a: &'a Value) -> Option<Cow<'a, Value>> {
+    pub(crate) fn has_parent(&self) -> bool {
+        for op in &self.segments {
+            let result = match op {
+                Segment::Dot(_, RawSelector::Parent(_))
+                | Segment::Recursive(_, Some(RawSelector::Parent(_)))
+                | Segment::Bracket(_, BracketSelector::Parent(_)) => true,
+                Segment::Bracket(_, BracketSelector::Path(p)) => p.has_parent(),
+                Segment::Bracket(_, BracketSelector::Filter(f)) => f.has_parent(),
+                _ => false,
+            };
+
+            if result {
+                return true;
+            }
+        }
+        false
+    }
+
+    fn eval_expr<'a>(&self, ctx: &EvalCtx<'a, '_>, a: &'a Value) -> Option<Cow<'a, Value>> {
         let relative = match self.kind {
             PathKind::Root(_) => false,
             PathKind::Relative(_) => true,
@@ -230,7 +241,7 @@ impl SubPath {
 
         let new_root = if relative { a } else { ctx.root() };
 
-        let mut new_ctx = EvalCtx::new_parents(new_root, ctx.all_parents().clone());
+        let mut new_ctx = EvalCtx::new_parents(new_root, ctx.all_parents());
         for op in &self.segments {
             op.eval(&mut new_ctx);
         }
@@ -249,33 +260,33 @@ impl SubPath {
         }
     }
 
-    fn eval_match(&self, ctx: &mut EvalCtx<'_>) {
+    fn eval_match(&self, ctx: &mut EvalCtx<'_, '_>) {
         let relative = match self.kind {
             PathKind::Root(_) => false,
             PathKind::Relative(_) => true,
         };
 
-        ctx.apply_matched(|ctx, a| {
+        ctx.set_matched(ctx.apply_matched_ref(|ctx, a| {
             let new_root = if relative { a } else { ctx.root() };
 
-            let mut new_ctx = EvalCtx::new_parents(new_root, ctx.all_parents().clone());
+            let mut new_ctx = EvalCtx::new_parents(new_root, ctx.all_parents());
             for op in &self.segments {
                 op.eval(&mut new_ctx);
             }
-            let matched = new_ctx.into_matched();
 
-            let matched = if self.tilde.is_some() {
-                matched
-                    .into_iter()
-                    .map(|a| Cow::Owned(ctx.idx_of(a).unwrap().into()))
-                    .collect::<Vec<_>>()
-            } else {
-                matched.into_iter().map(Cow::Borrowed).collect()
-            };
+            let id = self.tilde.is_some();
 
-            matched
+            new_ctx
+                .into_matched()
                 .into_iter()
-                .flat_map(|mat| match a {
+                .map(move |a| {
+                    if id {
+                        Cow::Owned(ctx.idx_of(a).unwrap().into())
+                    } else {
+                        Cow::Borrowed(a)
+                    }
+                })
+                .flat_map(move |mat| match a {
                     Value::Array(v) => {
                         let idx = match &*mat {
                             Value::Number(n) => idx_handle(n.as_i64().unwrap(), v),
@@ -294,37 +305,38 @@ impl SubPath {
                     }
                     _ => None,
                 })
-                .collect::<Vec<_>>()
-        });
+        }));
     }
 }
 
 impl Filter {
-    fn eval(&self, ctx: &mut EvalCtx<'_>) {
-        ctx.apply_matched(|ctx, a| match a {
-            Value::Array(v) => v
-                .iter()
-                .filter(|&a| {
-                    self.inner
-                        .eval_expr(ctx, a)
-                        .map_or(false, |c| c.as_bool() == Some(true))
-                })
-                .collect(),
-            Value::Object(m) => m
-                .values()
-                .filter(|&a| {
-                    self.inner
-                        .eval_expr(ctx, a)
-                        .map_or(false, |c| c.as_bool() == Some(true))
-                })
-                .collect(),
-            _ => vec![],
-        });
+    fn has_parent(&self) -> bool {
+        self.inner.has_parent()
+    }
+
+    fn eval(&self, ctx: &mut EvalCtx<'_, '_>) {
+        ctx.set_matched(ctx.apply_matched_ref(|ctx, a| {
+            a.iter().filter(|&a| {
+                self.inner
+                    .eval_expr(ctx, a)
+                    .map_or(false, |c| c.as_bool() == Some(true))
+            })
+        }));
     }
 }
 
 impl FilterExpr {
-    fn eval_expr<'a>(&self, ctx: &EvalCtx<'a>, val: &'a Value) -> Option<Cow<'a, Value>> {
+    fn has_parent(&self) -> bool {
+        match self {
+            FilterExpr::Unary(_, inner) => inner.has_parent(),
+            FilterExpr::Binary(left, _, right) => left.has_parent() || right.has_parent(),
+            FilterExpr::Parens(_, inner) => inner.has_parent(),
+            FilterExpr::Path(p) => p.has_parent(),
+            _ => false,
+        }
+    }
+
+    fn eval_expr<'a>(&self, ctx: &EvalCtx<'a, '_>, val: &'a Value) -> Option<Cow<'a, Value>> {
         match self {
             FilterExpr::Unary(op, inner) => {
                 let inner = inner.eval_expr(ctx, val)?;
