@@ -108,6 +108,34 @@ fn test_delete_in_try_replace() {
     assert_eq!(result, json!({"list": []}));
 }
 
+#[test]
+fn root_subpath_after_descent() {
+    let json = json!({"id": "foo", "a": {"b": {"c": {"id": "baz", "foo": 1, "bar": 2, "baz": 3}}}});
+    let result = find("$.a.b.c[$.id]", &json).unwrap();
+
+    let expected = [&json.as_object().unwrap()["a"].as_object().unwrap()["b"]
+        .as_object()
+        .unwrap()["c"]
+        .as_object()
+        .unwrap()["foo"]];
+
+    assert_eq!(result, expected);
+}
+
+#[test]
+fn relative_subpath_after_descent() {
+    let json = json!({"id": "foo", "a": {"b": {"c": {"id": "baz", "foo": 1, "bar": 2, "baz": 3}}}});
+    let result = find("$.a.b.c[@.id]", &json).unwrap();
+
+    let expected = [&json.as_object().unwrap()["a"].as_object().unwrap()["b"]
+        .as_object()
+        .unwrap()["c"]
+        .as_object()
+        .unwrap()["baz"]];
+
+    assert_eq!(result, expected);
+}
+
 #[test]
 fn dot_notation_after_recursive_descent() {
     let json = json!({
@@ -153,12 +181,9 @@ fn bracket_notation_after_recursive_descent() {
 #[test]
 fn parent_after_dot_notation() {
     let json = json!({"a": {"b": true}});
-    let result = find("$.a.b.^", &json)
-        .unwrap();
+    let result = find("$.a.b.^", &json).unwrap();
 
-    let expected = vec![
-        &json.as_object().unwrap()["a"],
-    ];
+    let expected = vec![&json.as_object().unwrap()["a"]];
 
     assert_eq!(result, expected);
 }
